/*         ______   ___    ___
 *        /\  _  \ /\_ \  /\_ \
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      Low-level file hooking mechanism.
 *
 *      By Ben Davis.
 *
 *      See readme.txt for copyright information.
 */


#ifndef ALLEGRO_FILEHOOK_H
#define ALLEGRO_FILEHOOK_H

#ifdef __cplusplus
   extern "C" {
#endif

#include "base.h"

AL_FUNC(int, al_open, (AL_CONST char *filename, int mode));

typedef struct AL_FILEHOOKS
{
   AL_METHOD(int, close, (void *data));
   AL_METHOD(long, lseek, (void *data, long offset, int whence));
   AL_METHOD(long, read, (void *data, void *buffer, long length));
   AL_METHOD(int, write, (void *data, AL_CONST void *buffer, long count));
} AL_FILEHOOKS;

typedef AL_METHOD(AL_FILEHOOKS *, AL_OPEN_HOOK, (AL_CONST char *file, int mode, void **data));

AL_FUNC(int, al_add_open_hook, (AL_OPEN_HOOK hook));
AL_FUNC(void, al_remove_open_hook, (AL_OPEN_HOOK hook));

AL_FUNC(int, al_virtualise_fd, (int real_fd));

AL_FUNC(int, al_filehook_open, (AL_CONST char *file, int mode));
AL_FUNC(int, al_filehook_dup, (int old_fd));
AL_FUNC(int, al_filehook_close, (int fd));
AL_FUNC(long, al_filehook_lseek, (int fd, long offset, int whence));
AL_FUNC(long, al_filehook_read, (int fd, void *buffer, long length));
AL_FUNC(int, al_filehook_write, (int fd, AL_CONST void *buffer, long count));

#ifdef __cplusplus
   }
#endif

#endif          /* ifndef ALLEGRO_FILEHOOK_H */

